@@ -11,10 +11,13 @@ import sys
 import time
 
 from . import exceptions
-from .log import Logger
+from .log import prep_generic_logger
 from .url import URLs
 from .common import helpers
+from logging import getLogger, RootLogger
+from typing import List
 
+log = getLogger(__name__)
 cached_obj = namedtuple('CachedObject', ['time', 'value'])
 
 
@@ -24,7 +27,6 @@ class LogicHubAPI:
     USER_AGENT_STRING = "LHUB CLI"
     http_timeout_default = 120
     verify_ssl = True
-    log: Logger = None
     last_response_status = None
     last_response_text = None
 
@@ -48,7 +50,14 @@ class LogicHubAPI:
     __credentials = {}
     __session_cookie = None
 
-    def __init__(self, hostname, api_key=None, username=None, password=None, verify_ssl=True, cache_seconds=None, default_timeout=None, **kwargs):
+    def __init__(self, hostname, api_key=None, username=None, password=None,
+                 verify_ssl=True, cache_seconds=None, default_timeout=None,
+                 logger: RootLogger = None, log_level=None, **kwargs):
+        global log
+        if logger:
+            log = logger
+        else:
+            prep_generic_logger(level=log_level)
         if not hostname:
             raise exceptions.validation.InputValidationError(input_var=None, action_description="instantiation", message="No hostname provided")
         if not api_key and not (username and password):
@@ -64,8 +73,6 @@ class LogicHubAPI:
         if self.cache_seconds is None or self.cache_seconds < 0:
             self.cache_seconds = 0
 
-        if not self.log:
-            self.log = Logger()
         if default_timeout:
             self.http_timeout_default = int(default_timeout)
         self._http_timeout_login = min(self._http_timeout_login, self.http_timeout_default)
@@ -75,7 +82,7 @@ class LogicHubAPI:
             self.verify_ssl = verify_ssl
         if not self.verify_ssl:
             # Disable certificate warnings
-            self.log.debug("Disabling SSL warnings...")
+            log.debug("Disabling SSL warnings...")
             disable_warnings(InsecureRequestWarning)
 
         if self.__api_key:
@@ -86,8 +93,8 @@ class LogicHubAPI:
 
         __init_version = self.kwargs.pop('init_version', None)
         self.url = URLs(hostname, init_version=__init_version)
+        log.debug(f"Starting session for server: {self.session_hostname}")
         self.__set_version(__init_version)
-        _ = atexit.register(self.close)
         self.formatted = FormattedObjects(api=self)
 
     def __enter__(self):
@@ -120,8 +127,7 @@ class LogicHubAPI:
             _ = self.get_version_info()
         return self.__version_info.value
 
-    @version_info.setter
-    def version_info(self, value):
+    def __set_version_info(self, value):
         self.__version_info = cached_obj(int(time.time()), value)
 
     @property
@@ -153,7 +159,7 @@ class LogicHubAPI:
             _ = self.get_version_info()
             return
         self.__version = cached_obj(int(time.time()), helpers.format_version(value))
-        self.log.debug(f"LogicHub version: {self.version}")
+        log.debug(f"LogicHub version: {self.version}")
         # Update the version attribute in self.url for use in other calls since the URLs class does not make any such calls on its own
         self.url._current_version = self.version
 
@@ -287,7 +293,7 @@ class LogicHubAPI:
         if self.auth_type == 'password':
             # if the response fails because the session has ended or the user has logged out, log back in and try once more
             if response.status_code == 401 and reauth and "not logged in" in response.text:
-                self.log.debug(f"Session expired; logging in and retrying")
+                log.debug(f"Session expired; logging in and retrying")
                 self.login(force_new_session=True)
                 response = request(method=method, url=url, verify=self.verify_ssl, timeout=timeout, cookies=self.cookies, **kwargs)
 
@@ -305,38 +311,39 @@ class LogicHubAPI:
             self.__session_cookie = None
 
         if self.__session_cookie:
-            self.log.debug("Already logged in; login skipped")
+            log.debug("Already logged in; login skipped")
             return
 
-        self.log.debug("Logging into LogicHub")
+        log.debug("Logging into LogicHub")
         login_response = self._http_request(url=self.url.login, method="POST", body=self.__credentials, timeout=self._http_timeout_login, reauth=False)
         self.session_cookie = login_response.cookies.get("PLAY_SESSION")
-        self.log.debug("Login successful")
+        log.debug("Login successful")
+        _ = atexit.register(self.close)
 
     def logout(self):
-        self.log.debug("Logout requested")
+        log.debug(f"Logout requested [{self.session_hostname}]")
         if self.auth_type != 'password':
             raise exceptions.LhBaseException('Login process is only used for password auth')
         if not self.session_cookie:
-            self.log.debug("Already logged out; logout skipped")
+            log.debug("Already logged out; logout skipped")
             return
 
-        self.log.debug("Issuing logout command")
+        log.debug("Issuing logout command")
         _response = self._http_request(url=self.url.logout, method="POST", test_response=False, reauth=False, timeout=self._http_timeout_login)
         try:
             _response.raise_for_status()
         except HTTPError as err:
-            self.log.warning(f"Logout failed with HTTP error: {str(err)}")
+            log.warning(f"Logout failed with HTTP error: {str(err)}")
         except Exception as err:
-            self.log.warning(f"Logout failed with UNKNOWN error: {repr(err)}")
+            log.warning(f"Logout failed with UNKNOWN error: {repr(err)}")
         else:
-            self.log.debug(f"Logout successful")
+            log.debug(f"Logout successful")
             self.session_cookie = None
 
     def close(self):
         """Steps to clean up on exit"""
         if self.auth_type == 'password':
-            self.log.debug("LogicHubAPI class exiting...")
+            log.debug("LogicHubAPI class exiting...")
             self.logout()
 
     # ToDo Revisit and finish this
@@ -374,7 +381,7 @@ class LogicHubAPI:
         limit = int(limit or 1000)
         offset = int(offset or 0)
         params = {"fields": "*", "pageSize": limit, "after": offset, "cachedOnly": "true"}
-        self.log.debug("Fetching batch results")
+        log.debug("Fetching batch results")
         response = self._http_request(url=self.url.batch_results_by_id.format(int(batch_id)), params=params)
         return response.json()
 
@@ -394,7 +401,7 @@ class LogicHubAPI:
             "after": int(offset or 0),
         }
         body = {"status": statuses or [], "excludeBatchesWithZeroEvents": exclude_empty_results}
-        self.log.debug("Fetching batches")
+        log.debug("Fetching batches")
         response = self._http_request(method="POST", url=self.url.stream_batches.format(stream_id), params=params, body=body)
         return response.json()
 
@@ -414,7 +421,7 @@ class LogicHubAPI:
                 connection_list.append(f"connection-{connection}")
 
         headers = {"Content-Type": "application/json"}
-        self.log.debug("Fetching connection status")
+        log.debug("Fetching connection status")
         response = self._http_request(method="POST", url=self.url.connection_status, headers=headers, body=connection_list)
         return response.json()
 
@@ -444,7 +451,7 @@ class LogicHubAPI:
             url = self.url.custom_list_data
             method = "GET"
             payload = None
-        self.log.debug("Fetching custom list data")
+        log.debug("Fetching custom list data")
         response = self._http_request(
             method=method,
             url=url,
@@ -459,12 +466,24 @@ class LogicHubAPI:
             raise exceptions.app.UnexpectedOutput("API response does not match the expected schema for listing custom lists")
         return results
 
-    # ToDo Token auth not supported as of 2022-03-09 (m92)
+    # ToDo Token auth not supported as of 2022-05-05 (m94)
+    def get_dashboard(self, dashboard_id):
+        log.debug("Fetching dashboard data")
+        response = self._http_request(url=self.url.dashboard.format(dashboard_id))
+        return response.json()
+
+    # ToDo Token auth not supported as of 2022-05-05 (m94)
     def get_dashboard_data(self, dashboard_id):
-        self.log.debug("Fetching dashboard data")
+        log.debug("Fetching dashboard data")
         response = self._http_request(url=self.url.dashboard_data.format(dashboard_id))
         return response.json()
 
+    # ToDo Token auth not supported as of 2022-05-05 (m94)
+    def get_widget_data(self, dashboard_id, widget_id):
+        log.debug("Fetching dashboard data")
+        response = self._http_request(url=self.url.widget_data.format(dashboard_id=dashboard_id, widget_id=widget_id))
+        return response.json()
+
     # ToDo STILL DOES NOT WORK WITH API AUTH AS OF M91
     def get_rule_set_by_name(self, name):
         rule_sets = self.list_rule_sets()
@@ -481,7 +500,7 @@ class LogicHubAPI:
         params = {
             "fields": "name,isPublic,rules[filter,values*1000,score]*1000",
         }
-        self.log.debug("Fetching rules")
+        log.debug("Fetching rules")
         response = self._http_request(url=self.url.rule_set.format(rule_set), params=params)
         results = response.json()
         try:
@@ -492,7 +511,7 @@ class LogicHubAPI:
 
     def get_stream_by_id(self, stream_id: int):
         headers = {"Accept": "application/json"}
-        self.log.debug("Fetching stream")
+        log.debug("Fetching stream")
         response = self._http_request(url=self.url.stream_by_id.format(stream_id), headers=headers, test_response=False)
         if response.status_code == 400 and 'Cannot find entity for id StreamId' in response.text:
             raise exceptions.app.StreamNotFound(stream_id)
@@ -501,7 +520,7 @@ class LogicHubAPI:
         return response.json()
 
     def get_version_info(self):
-        self.log.debug("Fetching LogicHub version info")
+        log.debug("Fetching LogicHub version info")
         try:
             response = self._http_request(
                 url=self.url.version,
@@ -512,13 +531,13 @@ class LogicHubAPI:
             raise exceptions.LhBaseException("LogicHub version could not be established")
         else:
             # Update version information any time this api call is run successfully
-            self.version_info = response_dict
+            self.__set_version_info(response_dict)
             self.__set_version(self.version_info)
         return response_dict
 
     def get_workflow_by_id(self, workflow_id: int):
         assert isinstance(workflow_id, int), "Workflow ID must be an integer"
-        self.log.debug("Fetching workflow")
+        log.debug("Fetching workflow")
         response = self._http_request(method="GET", url=self.url.case_status_workflow_by_id.format(workflow_id))
         return response.json()
 
@@ -536,7 +555,7 @@ class LogicHubAPI:
                 "score": score
             }
         }
-        self.log.debug("Issuing new scoring rule")
+        log.debug("Issuing new scoring rule")
         response = self._http_request(url=self.url.rule_set.format(rule_set), method="POST", body=body, headers=headers)
         results = response.json()
         try:
@@ -546,7 +565,7 @@ class LogicHubAPI:
         return results
 
     def export_playbook(self, playbook_id, test_response=True):
-        self.log.debug(f"Exporting playbook {playbook_id}")
+        log.debug(f"Exporting playbook {playbook_id}")
         response = self._http_request(url=self.url.flow_export.format(playbook_id), test_response=test_response)
         return response.json()
 
@@ -558,7 +577,7 @@ class LogicHubAPI:
         :return:
         """
         limit = int(limit if limit and limit > 0 else 999999999)
-        self.log.debug("Executing LogicHub command")
+        log.debug("Executing LogicHub command")
         response = self._http_request(
             url=self.url.command_execute,
             method="POST",
@@ -569,16 +588,16 @@ class LogicHubAPI:
         try:
             result_dict = response.json()
         except json.decoder.JSONDecodeError:
-            self.log.fatal(f"Failed to load API response as JSON. Status code: {response.status_code} Response text: {response.text}")
+            log.fatal(f"Failed to load API response as JSON. Status code: {response.status_code} Response text: {response.text}")
             sys.exit(1)
 
         errors = result_dict.pop("errors", [])
         if errors:
-            self.log.alert(f"Request was successful, but command failed with errors")
-            self.log.debug(f"Full error list: {json.dumps(errors)}")
-            self.log.debug(f"Other error response data: {json.dumps(result_dict)}")
+            log.warning(f"Request was successful, but command failed with errors")
+            log.debug(f"Full error list: {json.dumps(errors)}")
+            log.debug(f"Other error response data: {json.dumps(result_dict)}")
             try:
-                self.log.alert(f"Direct link to command: {self.url.command.format(errors[0]['details']['flowId'])}")
+                log.warning(f"Direct link to command: {self.url.command.format(errors[0]['details']['flowId'])}")
             except KeyError:
                 pass
             for error in errors:
@@ -591,15 +610,15 @@ class LogicHubAPI:
                     #   <<(lhub_file_id,None)>> does not exist in the table.", "flowId": "flow-1019", "nodeId": "ac4ce79f-d6d4-40fc-b855-77086fca92d5",
                     #   "causes": ["lql.LqlValidator$OperatorException: [Error Executing Operator: select]", "java.lang.IllegalArgumentException:
                     #   <<(lhub_file_id,None)>> does not exist in the table."]}
-                    self.log.debug(f"Command error details: {error_details}")
+                    log.debug(f"Command error details: {error_details}")
 
                 if error:
                     # [DEBUG] [0x10f00cef0] Additional error fields: {"cause": "lql.LqlValidator$OperatorException: [Error Executing Operator: select]"}
-                    self.log.debug(f"Additional error fields: {json.dumps(error)}")
+                    log.debug(f"Additional error fields: {json.dumps(error)}")
 
                 # [FATAL] [0x11dc8b400] Command returned error (SparkRuntimeException): [Execution Error] Executing node final_output failed.
                 # [Error Executing Operator: select]. <<(lhub_file_id,None)>> does not exist in the table.
-                self.log.error(f"Command returned error ({error_type}): {error_msg}")
+                log.error(f"Command returned error ({error_type}): {error_msg}")
             sys.exit(1)
 
         try:
@@ -610,7 +629,7 @@ class LogicHubAPI:
                 message += json.dumps(response.json(), indent=2)
             except (json.decoder.JSONDecodeError, ValueError, TypeError):
                 message += response.text
-            self.log.fatal(message)
+            log.fatal(message)
         return response.json()
 
     def list_baselines(self, limit=None, offset=0, filters=None):
@@ -625,7 +644,7 @@ class LogicHubAPI:
         limit = int(limit if limit and limit > 0 else 999999999)
         params = {"libraryView": "all"}
         body = {"filters": filters or [], "offset": offset or 0, "pageSize": limit, "sortColumn": "name", "sortOrder": "ASC"}
-        self.log.debug("Fetching baselines")
+        log.debug("Fetching baselines")
         response = self._http_request(method="POST", url=self.url.baselines, params=params, body=body)
         return response.json()
 
@@ -641,7 +660,7 @@ class LogicHubAPI:
         limit = int(limit if limit and limit > 0 else 999999999)
         params = {"libraryView": "all"}
         body = {"filters": filters or [], "offset": offset or 0, "pageSize": limit, "sortColumn": "name", "sortOrder": "ASC"}
-        self.log.debug("Fetching commands")
+        log.debug("Fetching commands")
         response = self._http_request(method="POST", url=self.url.commands, params=params, body=body)
         return response.json()
 
@@ -658,7 +677,7 @@ class LogicHubAPI:
         params = {"libraryView": "all"}
         headers = {"Content-Type": "application/json"}
         body = {"filters": filters or [], "offset": offset, "pageSize": limit, "sortColumn": "name", "sortOrder": "ASC"}
-        self.log.debug("Fetching connections")
+        log.debug("Fetching connections")
         response = self._http_request(method="POST", url=self.url.connections, headers=headers, params=params, body=body)
         return response.json()
 
@@ -678,7 +697,7 @@ class LogicHubAPI:
         if search_text:
             filters.append({"searchText": search_text})
 
-        self.log.debug("Fetching custom lists")
+        log.debug("Fetching custom lists")
         response = self._http_request(
             method="POST",
             url=self.url.custom_lists,
@@ -702,12 +721,12 @@ class LogicHubAPI:
 
     # ToDo Token auth not supported as of 2022-03-09 (m92)
     def list_dashboards(self):
-        self.log.debug("Fetching dashboards")
+        log.debug("Fetching dashboards")
         response = self._http_request(url=self.url.dashboards)
         return response.json()
 
     def list_dashboards_with_widgets(self):
-        self.log.debug("Fetching dashboards with widgets")
+        log.debug("Fetching dashboards with widgets")
         response = self._http_request(url=self.url.dashboards_and_widgets)
         return response.json()
 
@@ -720,7 +739,7 @@ class LogicHubAPI:
         # Not sure when this one stopped working, but it worked somewhere in at least m66 and didn't work any more in m70
         limit = int(limit if limit and limit > 0 else 999999999)
         params = {"limit": limit}
-        self.log.debug("Fetching event types")
+        log.debug("Fetching event types")
         response = self._http_request(url=self.url.event_types, params=params)
         results = response.json()
         if results.get("result"):
@@ -740,7 +759,7 @@ class LogicHubAPI:
         params = {"libraryView": "all"}
         # Confirm the right term for "name" as the sort column and change this sort order
         body = {"filters": [], "pageSize": limit, "offset": offset or 0, "sortColumn": "lastUpdated", "sortOrder": "DESC"}
-        self.log.debug("Fetching event types")
+        log.debug("Fetching event types")
         response = self._http_request(method="POST", url=self.url.event_types, params=params, body=body)
         results = response.json()
         if results.get("result"):
@@ -763,18 +782,18 @@ class LogicHubAPI:
 
     def list_fields(self, params: dict = None, **kwargs):
         params = params or {"systemFields": "true", "pageSize": 9999, "after": 0}
-        self.log.debug("Fetching fields")
+        log.debug("Fetching fields")
         response = self._http_request(method="GET", url=self.url.fields, params=params, **kwargs)
         self.fields = response.json()
         return response.json()
 
     def list_integrations(self):
-        self.log.debug("Fetching integrations")
+        log.debug("Fetching integrations")
         response = self._http_request(method="GET", url=self.url.integrations)
         return response.json()
 
     def list_ml_models(self):
-        self.log.debug("Fetching ML models")
+        log.debug("Fetching ML models")
         response = self._http_request(
             method="GET",
             url=self.url.ml_models,
@@ -783,7 +802,7 @@ class LogicHubAPI:
         return response.json()
 
     def list_modules(self):
-        self.log.debug("Fetching modules")
+        log.debug("Fetching modules")
         response = self._http_request(
             method="GET",
             url=self.url.modules,
@@ -810,7 +829,7 @@ class LogicHubAPI:
         }
         if search_string:
             body['filters'] = [{"searchText": search_string}]
-        self.log.debug("Fetching notebooks")
+        log.debug("Fetching notebooks")
         response = self._http_request(
             url=self.url.notebooks_list,
             method="POST",
@@ -833,7 +852,7 @@ class LogicHubAPI:
         """
         limit = int(limit if limit and limit > 0 else 999999999)
         body = {"filters": [], "offset": offset or 0, "pageSize": limit, "sortColumn": "name", "sortOrder": "ASC"}
-        self.log.debug("Fetching playbooks")
+        log.debug("Fetching playbooks")
         response = self._http_request(
             url=self.url.playbooks_list,
             method="POST",
@@ -854,7 +873,7 @@ class LogicHubAPI:
             "fields": "name,isPublic",
             "pageSize": limit,
         }
-        self.log.debug("Fetching rule sets")
+        log.debug("Fetching rule sets")
         response = self._http_request(url=self.url.rule_sets, params=params)
         results = response.json()
         try:
@@ -863,12 +882,12 @@ class LogicHubAPI:
             raise exceptions.app.UnexpectedOutput("API response does not match the expected schema for listing rule sets")
         return results
 
-    def list_stream_states(self, stream_ids: list):
+    def list_stream_states(self, stream_ids: List[int]):
         new_stream_id_list = []
         for s in stream_ids:
             new_stream_id_list.append(f'stream-{s}')
         body = {"streams": new_stream_id_list}
-        self.log.debug("Fetching stream states")
+        log.debug("Fetching stream states")
         response = self._http_request(method="POST", url=self.url.stream_states, body=body)
         return response.json()
 
@@ -898,17 +917,17 @@ class LogicHubAPI:
             "sortOrder": "ASC"
         }
 
-        self.log.debug("Fetching streams")
+        log.debug("Fetching streams")
         response = self._http_request(method="POST", url=self.url.streams, body=body_dict, headers=headers, params=params)
         return response.json()
 
-    def list_user_groups(self, limit=None, hide_inactive=False):
+    def list_user_groups(self, limit=None, hide_inactive=True):
         limit = limit if limit and isinstance(limit, int) else 99999
         params = {"pageSize": limit, "after": 0}
         body = {"filters": []}
         if hide_inactive:
             body['filters'].append({"hideDeleted": True})
-        self.log.debug("Fetching user groups")
+        log.debug("Fetching user groups")
         response = self._http_request(
             url=self.url.user_groups,
             method="POST",
@@ -918,13 +937,13 @@ class LogicHubAPI:
         )
         return response.json()
 
-    def list_users(self, limit=None, hide_inactive=False):
+    def list_users(self, limit=None, hide_inactive=True):
         limit = limit if limit and isinstance(limit, int) else 99999
         params = {"pageSize": limit, "after": 0}
         body = {"filters": []}
         if hide_inactive:
             body['filters'].append({"hideInactive": True})
-        self.log.debug("Fetching users")
+        log.debug("Fetching users")
         response = self._http_request(
             url=self.url.users,
             method="POST",
@@ -935,7 +954,7 @@ class LogicHubAPI:
         return response.json()
 
     def list_workflows(self):
-        self.log.debug("Fetching workflows")
+        log.debug("Fetching workflows")
         response = self._http_request(method="GET", url=self.url.case_status_list_workflows)
         return response.json()
 
@@ -950,12 +969,12 @@ class LogicHubAPI:
             batch_id = int(re.sub(r'\D+', '', str(batch_id)))
         headers = {"Content-Type": "application/json"}
         body = {"method": "reschedule", "parameters": {}}
-        self.log.debug("Issuing command to reprocess batch")
+        log.debug("Issuing command to reprocess batch")
         response = self._http_request(method="POST", url=self.url.batch_reprocess.format(batch_id), headers=headers, body=body)
         return response.json()
 
     def me(self):
-        self.log.debug("Fetching current user info")
+        log.debug("Fetching current user info")
         response = self._http_request(method="GET", url=self.url.me, timeout=30)
         result = response.json()
         if not all((
@@ -977,11 +996,11 @@ class LogicHubAPI:
             # Username will only exist already if password auth is used. In case of API token auth, capture and record the username.
             if self.__username != result['result']['username']:
                 self.__username = result['result']['username']
-                self.log.debug(f"Current user updated to {self.__username} [ID: {self.__user_id}, role: {self.__user_role}]")
+                log.debug(f"Current user updated to {self.__username} [ID: {self.__user_id}, role: {self.__user_role}]")
         return result
 
     def update_current_user_preferences(self, preferences):
-        self.log.debug("Patching user (updating preferences)")
+        log.debug("Patching user (updating preferences)")
         response = self._http_request(method="PATCH", url=self.url.user.format(self.session_user_id), body={"preferences": preferences})
         return response.json()
 
@@ -1006,7 +1025,7 @@ class LogicHubAPI:
     #
     #     field_id = self.formatted.system_field_lh_linked_alerts.get('id')
     #     body = {"fields": [{"id": field_id, "value": alert_ids}]}
-    #     self.log.debug("Linking alert to case")
+    #     log.debug("Linking alert to case")
     #     response = self._http_request(
     #         method="PATCH",
     #         url=self.url.case_update_linked_alerts.format(case_id=case_id),
@@ -1015,7 +1034,7 @@ class LogicHubAPI:
     #     output = response.json()
 
     def cases_get_prefix(self):
-        self.log.debug("Fetching case prefix")
+        log.debug("Fetching case prefix")
         response = self._http_request(
             url=self.url.cases_get_prefix,
             headers={"Content-Type": "application/json"}
@@ -1029,7 +1048,7 @@ class LogicHubAPI:
             "id": helpers.format_case_id_with_prefix(case_id, self.case_prefix),
             "key": "case",
         }
-        self.log.debug("Fetching attached notebooks")
+        log.debug("Fetching attached notebooks")
         response = self._http_request(
             url=self.url.notebooks_attached,
             method="POST",
@@ -1054,7 +1073,7 @@ class LogicHubAPI:
                 "id": helpers.format_case_id_with_prefix(case_id, self.case_prefix),
             }
         }
-        self.log.debug("Updating attached notebooks")
+        log.debug("Updating attached notebooks")
         response = self._http_request(
             url=self.url.notebooks_attach,
             method="PATCH",
@@ -1068,7 +1087,7 @@ class LogicHubAPI:
         alert_id = helpers.format_alert_id(alert_id)
         url = self.url.alert_fetch.format(alert_id)
         # Send request, but disable immediate testing in order to do specialized testing first
-        self.log.debug("Fetching alert")
+        log.debug("Fetching alert")
         response = self._http_request(method="GET", url=url, test_response=False, **kwargs)
         if return_raw:
             return response
@@ -1081,7 +1100,7 @@ class LogicHubAPI:
         if query:
             # First confirm that the query is valid
             body = {"queryType": "advanced", "query": query}
-            self.log.debug("Validating search syntax")
+            log.debug("Validating search syntax")
             response = self._http_request(
                 url=self.url.alerts_search_validate,
                 method="POST",
@@ -1109,7 +1128,7 @@ class LogicHubAPI:
             "sortOrder": "desc",
             "query": query
         }
-        self.log.debug("Searching alerts")
+        log.debug("Searching alerts")
         response = self._http_request(
             url=self.url.alerts_search_advanced,
             method="POST",
@@ -1121,14 +1140,14 @@ class LogicHubAPI:
     def stream_pause(self, stream_id, **kwargs):
         # Sanitize stream ID
         body = [f"stream-{helpers.format_stream_id(stream_id)}"]
-        self.log.debug("Pausing stream")
+        log.debug("Pausing stream")
         response = self._http_request(method="POST", url=self.url.stream_pause, body=body, **kwargs)
         return response.json()
 
     def stream_resume(self, stream_id, **kwargs):
         # Sanitize stream ID
         body = [f"stream-{helpers.format_stream_id(stream_id)}"]
-        self.log.debug("Resuming stream")
+        log.debug("Resuming stream")
         response = self._http_request(method="POST", url=self.url.stream_resume, body=body, **kwargs)
         return response.json()
 
@@ -1139,12 +1158,12 @@ class LogicHubAPI:
             "userGroupIds": group_ids,
             "authenticationType": authentication_type or "password"
         }
-        self.log.debug(f"Creating user: {username}")
+        log.debug(f"Creating user: {username}")
         response = self._http_request(method="POST", url=self.url.user_create, body=body)
         return response.json()
 
     def delete_user(self, user_ids: list):
-        self.log.debug(f"Deleting users by id: {', '.join([str(i) for i in user_ids])}")
+        log.debug(f"Deleting users by id: {', '.join([str(i) for i in user_ids])}")
         response = self._http_request(method="POST", url=self.url.user_delete, body=user_ids)
         return response.json()
 
