@@ -3,23 +3,23 @@ import re
 from copy import deepcopy
 
 from .api import LogicHubAPI
-from .log import Logger
 from .common import helpers
 from .common.time import epoch_time_to_str
-from .exceptions.app import BaseAppError, UserGroupNotFound, UserNotFound
+from .exceptions.app import BaseAppError, UserGroupNotFound, UserNotFound, UnexpectedOutput
 from .exceptions.validation import InputValidationError, ResponseValidationError
 from .exceptions.formatting import InvalidWorkflowIdFormat
+from .log import prep_generic_logger
+from logging import getLogger, RootLogger
+from typing import List, Union
+
+log = getLogger(__name__)
 
 
 class Actions:
-    log = None
 
     def __init__(self, api: LogicHubAPI):
         self.__api = api
 
-        # If this class has not been given a logger by the time it is instantiated, inherit the same one from LogicHubAPI
-        self.log = self.log or self.__api.log
-
     @staticmethod
     def __enrich_alert_data(alert: dict, included_standard_fields=None, included_additional_fields=None):
         # Verify and sanitize inputs
@@ -74,7 +74,8 @@ class Actions:
         response = self.list_playbooks()
         return {p["id"]["id"]: p["name"] for p in response}
 
-    def __reformat_cmd_results(self, response_dict, drop_hidden_columns=True):
+    @staticmethod
+    def __reformat_cmd_results(response_dict, drop_hidden_columns=True):
         full_result = response_dict.copy()
         result_raw = full_result["result"]
         warnings = full_result["result"].get("warnings")
@@ -86,7 +87,7 @@ class Actions:
             result_output = [{k: v for k, v in _result['fields'].items()} for _result in result_with_schema]
 
         for _warning in warnings:
-            self.log.debug(f"WARNING RETURNED: {_warning}")
+            log.warning(f"Warning returned: {_warning}")
         return result_output, warnings
 
     def execute_command(self, command_name, input_dict, reformat=True, result_limit=None):
@@ -185,6 +186,22 @@ class Actions:
         _ = self._result_dict_has_schema(result, "result", raise_errors=True, action_description="get workflow by ID")
         return result["result"]
 
+    def list_stream_states(self, stream_ids, include_recent_batches=True, return_as_simple_dict=False):
+        stream_ids = helpers.format_stream_id(stream_ids)
+        result = self.__api.list_stream_states(stream_ids=stream_ids)
+        _ = self._result_dict_has_schema(result, "result", "streams", raise_errors=True, action_description="get stream states")
+        results = result['result']['streams']
+        if return_as_simple_dict:
+            return {r["streamId"]: r["status"] for r in results}
+        if not include_recent_batches:
+            for n in range(len(results)):
+                _stream_id = results[n]['streamId']
+                if 'data' in results[n]:
+                    del results[n]['data']
+                else:
+                    log.warning(f"Expected recent batches in 'data' key of stream state, but no such key was found for stream {_stream_id}")
+        return results
+
     def list_baselines(self):
         result = self.__api.list_baselines()
         _ = self._result_dict_has_schema(result, "result", "data", "data", raise_errors=True, action_description="list baselines")
@@ -216,7 +233,7 @@ class Actions:
         result = self.__api.list_commands(filters=filters, limit=limit, offset=0)
         _ = self._result_dict_has_schema(result, "result", "data", "data", action_description="list commands", raise_errors=True)
         results = result["result"]["data"]
-        self.log.debug(f"{len(results)} commands found")
+        log.debug(f"{len(results)} command{'s' if len(results) != 1 else ''} found")
         if simple_format:
             results = [self.__reformat_command_simple(r) for r in results['data']]
         return results
@@ -325,10 +342,38 @@ class Actions:
             output = {f['name']: {k: v for k, v in f.items() if k != 'name'} for f in output}
         return output
 
-    def list_streams(self, search_text: str = None, filters: list = None, limit: int = None, offset: int = 0):
+    def list_streams(self, search_text: str = None, filters: list = None, limit: int = None, offset: int = 0, verify_stream_states=False):
+        """
+        List all streams (or streams matching a search filter)
+
+        :param search_text: Partial or full name of the streams to return
+        :param filters: Other search filters (see the streams API)
+        :param limit: Limit the number of results to return
+        :param offset: For pagination, provide the page number of results to return
+        :param verify_stream_states: The streams API does not wait for stream status
+        calculation to complete unless status is part of the search filter. Enabling
+        this option will fetch the true state for every stream returned before returning results.
+        :param return_as_simple_dict: Return a simple dict instead, with stream IDs as keys and their respective statuses as values.
+        :return:
+        """
         result = self.__api.list_streams(search_text=search_text, filters=filters, limit=limit, offset=offset)
+        # ToDo get accurate states with actions.list_stream_states
+        # ToDo file a bug for the streams api returning incorrect states
         _ = self._result_dict_has_schema(result, "result", "data", "data", raise_errors=True, action_description="list streams")
-        return result["result"]["data"]["data"]
+        results = result["result"]["data"]["data"]
+        if verify_stream_states is True:
+            id_list = helpers.format_stream_id(results)
+            states = self.list_stream_states(stream_ids=id_list, include_recent_batches=False, return_as_simple_dict=True)
+            for r in results:
+                if not r.get('id') or not r['id'].get('id'):
+                    raise UnexpectedOutput(f"stream dict did not match expected format: {r}")
+                _id_str = f"stream-{r['id']['id']}"
+                _id_str = f"stream-{r['id']['id']}"
+                if not states.get(_id_str):
+                    raise UnexpectedOutput(f"{_id_str} was returned in the stream search but was not present in state search results")
+                r['streamStatus'] = states[_id_str]
+
+        return results
 
     def list_user_groups(self, hide_inactive=False):
         result = self.__api.list_user_groups(hide_inactive=hide_inactive)
@@ -355,7 +400,8 @@ class Actions:
         result = self.__api.list_users(hide_inactive=hide_inactive)
         _ = self._result_dict_has_schema(result, "result", "data", action_description="list users", raise_errors=True)
         results = result["result"]
-        self.log.debug(f"{len(results['data'])} users found")
+        result_count = len(results['data'])
+        log.debug(f"{result_count} user{'s' if result_count != 0 else ''} found")
         if simple_format:
             results = [self.__reformat_user_simple(result) for result in results['data']]
         return results
@@ -466,13 +512,13 @@ class Actions:
                     if g not in _groups:
                         raise UserGroupNotFound(input_var=g)
                     _group_id = helpers.format_user_group_id(_groups[g])
-                    self.log.debug(f"Translated group {g} to ID {_group_id}")
+                    log.debug(f"Translated group {g} to ID {_group_id}")
                     group_ids.append(_group_id)
             else:
                 group_ids = _groups["Everyone"]
 
         kwargs = {"username": username, "email": email, "authentication_type": authentication_type, "group_ids": group_ids}
-        self.log.debug(f"Creating user: {json.dumps(kwargs)}")
+        log.debug(f"Creating user: {json.dumps(kwargs)}")
         response = self.__api.create_user(**kwargs)
         return response['result']
 
@@ -489,7 +535,7 @@ class Actions:
             if u not in _users:
                 raise UserNotFound(input_var=u)
             _user_id = helpers.format_user_id(_users[u])
-            self.log.debug(f"Translated user {u} to ID {_user_id}")
+            log.debug(f"Translated user {u} to ID {_user_id}")
             user_ids.append(_user_id)
         return self.delete_user_by_id(user_ids=user_ids)
 
@@ -499,22 +545,24 @@ class LogicHub:
 
     def __init__(
             self, hostname, api_key=None, username=None, password=None,
-            verify_ssl=True, cache_seconds=None, verify_api_auth=True,
-            logger: Logger = None, log_level=None, default_timeout=None,
-            **kwargs):
-        # If the LogicHubAPI class object has not been given a logger by the time this class is instantiated, set one for it
-        LogicHubAPI.log = LogicHubAPI.log or logger or Logger()
-        if log_level:
-            LogicHubAPI.log.log_level = log_level
-
-        LogicHubAPI.verify_ssl = self.verify_ssl
-
+            cache_seconds=None, verify_api_auth=True, default_timeout=None, logger: RootLogger = None, log_level=None, **kwargs):
+        global log
+        if logger:
+            log = logger
+        else:
+            prep_generic_logger(level=log_level)
+        self.hostname = hostname
         self.kwargs = kwargs
-        self.api = LogicHubAPI(hostname=hostname, api_key=api_key, username=username, password=password, verify_ssl=verify_ssl, cache_seconds=cache_seconds, default_timeout=default_timeout, **kwargs)
+        self.api = LogicHubAPI(
+            hostname=hostname, api_key=api_key, username=username, password=password,
+            cache_seconds=cache_seconds, default_timeout=default_timeout,
+            logger=logger, **kwargs)
         self.actions = Actions(self.api)
         if verify_api_auth:
             _ = self._verify_api_auth()
+        log.debug(f"LogicHub (lhub) session successfully initialized (hostname={self.hostname})")
 
     def _verify_api_auth(self):
         _ = self.api.me()
+        log.debug(f"Authentication successful (hostname={self.hostname})")
         return True
